@@ -1,11 +1,29 @@
 /*
- * Copyright (c) 2000 Qualcomm Incorporated.  All rights reserved.
+ * Copyright (c) 2005 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  */
 
 
 /* Changes:
+ *
+ *   02/08/01  [rcg]
+ *            - Now using p->nMaxBulls instead of NEWBULLCNT.
+ *
+ *   01/31/01  [rcg]
+ *            - Used saved-off original group ID instead of modified
+ *              group, which is normally set to group MAIL.
+ *
+ *   01/30/01  [rcg]
+ *            - Now using p->bGroup_bulls instead of GROUP_BULLS.
+ *            - Now using p->bUpdate_status_hdrs instead of NO_STATUS.
+ *
+ *   12/27/00  [rcg]
+ *            - Bulletins whose group is 'ALL' are sent to all users.
+ *            - Bulletins with invalid groups are sent to no users.
+ *
+ *   09/22/00  [rcg]
+ *            - Handle static mp->uidl_str.
  *
  *   07/07/00  [rcg]
  *            - Fixed incorrect calculation of visible_msg_num.
@@ -119,11 +137,6 @@ static int sequence = 0;
 static long timestamp;
 static char *ERRMESG = "[SYS/TEMP] Unable to copy bulletin %s to pop dropbox %s: %s (%d)";
 
-#ifdef USE_BULL_GROUPS
-#  define BULL_GROUPS 1
-#else
-#  define BULL_GROUPS 0
-#endif /* USE_BULL_GROUPS */
 
 typedef struct _file_list file_list;
 
@@ -145,16 +158,22 @@ int CopyOneBull ( POP *p, long bnum, char *name );
  * A group name is contained in the bulletin's filename.  If the user is 
  * a member of this group then he/she gets it.  The group name is the second
  * token in the bulletin's filename, for example, 
- * "001.staff.new_program_installed".  (Tokens are separated by periods.)  If
- * the second token doesn't match any group in the system (returned by 
- * getgrnam()) then the bulletin is delivered to every user.
+ * "001.staff.new_program_installed".  (Tokens are separated by periods.)
+ 
+ * If there is no second token, the bulletins is delivered to all users.
+ *
+ * If the second token doesn't match any group in the system (returned by 
+ * getgrnam()) then the bulletin is not delivered to any user.  However, a
+ * group name of 'ALL' is sent to all users. 
+ *
+ * If the user is not a member of any group, no bulletins are shown.
  */
-int 
-IsBulletinForMe ( POP *p, struct passwd *pwd, const char *bull_name )
+BOOL 
+IsBulletinForMe ( POP *p, const char *bull_name )
 
 {
     char           *token;
-    char            group_name [30];
+    char            group_name [ MAXUSERNAMELEN ];
     char           *ptr;
     char          **u_list;
     size_t          group_len = 0;
@@ -169,9 +188,10 @@ IsBulletinForMe ( POP *p, struct passwd *pwd, const char *bull_name )
      * there's no token -- bulletin is for everybody 
      */
     if ( token == NULL ) {
-        DEBUG_LOG1 ( p, "Can't find '.' in bull name; user gets bull: %.128s",
-                     bull_name );
-        return 1;
+        pop_log ( p, POP_NOTICE, HERE,
+                  "Can't find '.' in bulletin name; delivering to all users: %.128s",
+                  bull_name );
+        return TRUE;
     }
 
     ptr = group_name;
@@ -184,31 +204,41 @@ IsBulletinForMe ( POP *p, struct passwd *pwd, const char *bull_name )
     }
     *ptr = 0;
 
-    pwgrp = getgrgid ( pwd->pw_gid );
+    pwgrp = getgrgid ( p->orig_group );
     /*
      * If user is not a member of a group, no bulletins are shown.
      */
     if ( pwgrp == NULL ) {
         DEBUG_LOG3 ( p, "getgrgid failed for gid %u: %s (%d)",
-                    (unsigned) pwd->pw_gid, STRERROR(errno), errno );
-        return 0;
+                    (unsigned) p->orig_group, STRERROR(errno), errno );
+        return FALSE;
     }
 
     DEBUG_LOG2 ( p, "Primary gid for user %s is %u", 
-                 p->user, (unsigned) pwd->pw_gid );
+                 p->user, (unsigned) p->orig_group );
 
+    /*
+     * if group is 'ALL', deliver to all users.
+     */
+    if ( strcmp ( group_name, "ALL" ) == 0 ) {
+        DEBUG_LOG1 ( p, "Bulletin is for 'ALL'; delivering to all users: %.128s",
+                     bull_name );
+        return TRUE;
+    }
     /* 
      * now, let's find specified group 
      */
     grp = getgrnam ( group_name );
 
     /* 
-     * if there's no such group -- bulletin is for everybody 
+     * if there's no such group bulletin is not delivered to anyone 
      */
     if ( grp == NULL ) {
-        DEBUG_LOG2 ( p, "No such group: \"%.128s\"; user gets bull %.128s",
-                     group_name, bull_name );
-        return 1;
+        pop_log ( p, POP_NOTICE, HERE,
+                  "Non-existent group \"%.128s\" in bulletin; "
+                  "bulletin omitted: %.128s",
+                  group_name, bull_name );
+        return FALSE;
     }
     
     /*
@@ -220,9 +250,9 @@ IsBulletinForMe ( POP *p, struct passwd *pwd, const char *bull_name )
      * If user's primary group matches the bulletin's group, we matched
      * it the easy way.
      */
-    if ( pwd->pw_gid == b_group ) {
+    if ( p->orig_group == b_group ) {
         DEBUG_LOG1 ( p, "User's gid matches bulletin's: %s", bull_name );
-        return 1;
+        return TRUE;
     }
     
     /* 
@@ -239,14 +269,14 @@ IsBulletinForMe ( POP *p, struct passwd *pwd, const char *bull_name )
         if ( !strcmp ( *u_list, p->user ) ) {
             DEBUG_LOG2 ( p, "Found user %s in group list for %s",
                          p->user, bull_name );
-            return 1; /* user found! */
+            return TRUE; /* user found! */
             }
         u_list++;
     }
 
     DEBUG_LOG2 ( p, "User %s not found in group list for %s",
                  p->user, bull_name );
-    return 0;
+    return FALSE;
 }
 
 
@@ -295,6 +325,7 @@ insert_list ( POP *p, file_list *head, char *name )
     return( head );
 }
 
+
 /*
  *  pop_bull: Append any new bulletins to the end of the user's
  *  temporary maildrop. This function is called only from pop_dropcopy.
@@ -305,9 +336,9 @@ insert_list ( POP *p, file_list *head, char *name )
 int
 pop_bull ( POP *p, struct passwd *pwp )
 {
-   char             popBullName    [ 256 ];
+   char             popBullName    [ MAXFILENAME ];
 #ifdef POPBULLDIR
-   char             oldPopBullName [ 256 ];
+   char             oldPopBullName [ MAXFILENAME ];
 #endif
    FILE            *popBull                     = NULL;
    DIR             *dirp;
@@ -318,7 +349,6 @@ pop_bull ( POP *p, struct passwd *pwp )
    long             lastBullSent;
    char             buffer [ MAXMSGLINELEN ];
    int              res                         = POP_SUCCESS;
-   int              err                         = 0;
    int              save_count;
    int              save_visible_count;
    long             save_drop_size;
@@ -368,8 +398,8 @@ pop_bull ( POP *p, struct passwd *pwp )
       /*
        * We found a bulletin.  Insert it if it is for the user.
        */
-      if ( BULL_GROUPS == 0 ||
-           IsBulletinForMe ( p, pwp, dp->d_name ) 
+      if ( p->bGroup_bulls == FALSE ||
+           IsBulletinForMe ( p, dp->d_name ) 
          ) {
         if ( bullNumber > maxBullNumber ) 
             maxBullNumber = bullNumber;
@@ -385,10 +415,10 @@ pop_bull ( POP *p, struct passwd *pwp )
    /* 
     * Open the user's .popbull file and read the number of the last
     * bulletin sent to this user. If the file doesn't exist, create
-    * it and seed it with the current max bulletin number. Note that
-    * by default, new users do not get sent old bulletins, but if
-    * NEWBULLCNT is set, it specifies the maximum number of old
-    * bulletins seen by new users. 
+    * it and seed it with the current max bulletin number.
+    *
+    * p->nMaxBulls specifies the maximum number of old bulletins
+    * seen by new users. 
     */
 
 #ifdef BULLDB
@@ -421,12 +451,12 @@ pop_bull ( POP *p, struct passwd *pwp )
        
        if ( popBull == NULL ) {
            DEBUG_LOG1 ( p, "No .popbull file for %s", p->user );
-           lastBullSent = maxBullNumber - NEWBULLCNT;       
+           lastBullSent = maxBullNumber - p->nMaxBulls;
        }
        else
        if ( fgets ( buffer, MAXMSGLINELEN, popBull ) == NULL ) {
            DEBUG_LOG1 ( p, "Popbull file %s empty", popBullName );
-           lastBullSent = maxBullNumber - NEWBULLCNT; 
+           lastBullSent = maxBullNumber - p->nMaxBulls; 
            fclose ( popBull );
            popBull = NULL;
        }
@@ -434,7 +464,7 @@ pop_bull ( POP *p, struct passwd *pwp )
        if ( isdigit ( (int) *buffer ) == FALSE ) {
            DEBUG_LOG2 ( p, "Invalid data in file %s: \"%.128s\"",
                         popBullName, buffer );
-           lastBullSent = maxBullNumber - NEWBULLCNT;
+           lastBullSent = maxBullNumber - p->nMaxBulls;
            fclose ( popBull );
            popBull = NULL;
        }
@@ -489,7 +519,7 @@ pop_bull ( POP *p, struct passwd *pwp )
 
    if ( popBull == NULL ) {
        DEBUG_LOG1 ( p, "No .popbull file for %s", p->user );
-       lastBullSent = maxBullNumber - NEWBULLCNT;       
+       lastBullSent = maxBullNumber - p->nMaxBulls;
    }
    else
    if ( fgets ( buffer, MAXMSGLINELEN, popBull ) == NULL ) {
@@ -520,7 +550,7 @@ pop_bull ( POP *p, struct passwd *pwp )
    if ( !isdigit ( (int) *buffer ) ) {
        DEBUG_LOG2 ( p, "Invalid data in file %s: \"%.128s\"",
                     popBullName, buffer );
-       lastBullSent = maxBullNumber - NEWBULLCNT;
+       lastBullSent = maxBullNumber - p->nMaxBulls;
        fclose ( popBull );
        unlink ( popBullName );
        /*
@@ -583,6 +613,8 @@ pop_bull ( POP *p, struct passwd *pwp )
    if ( res == POP_SUCCESS ) {
 
 #ifdef BULLDB
+       int err = 0;
+
        bull_count.dptr  = (char *) &maxBullNumber;
        bull_count.dsize = sizeof(maxBullNumber);
        err = flock ( dbm_dirfno ( p->bull_db ), LOCK_EX );
@@ -665,8 +697,8 @@ CopyOneBull ( POP *p, long bnum, char *name )
 {
     FILE          *bull;
     char           buffer [ MAXMSGLINELEN ];
-    int            in_header            = 1;
-    int            first_line           = 1;
+    BOOL           in_header            = TRUE;
+    BOOL           first_line           = TRUE;
     int            nchar;
     int            msg_num;
     int            msg_vis_num          = 0;
@@ -708,15 +740,15 @@ CopyOneBull ( POP *p, long bnum, char *name )
     /* 
      * Just append the message, no Status or UIDL updates here 
      */
-#ifndef NO_STATUS
-    p->dirty = 1;
-#endif
+    if ( p->bUpdate_status_hdrs )
+        p->dirty = 1;
 
     mp = p->mlp + msg_num - 1;
 
     if ( ++msg_num > p->msg_count ) {
+        p->msg_count += ALLOC_MSGS;
         p->mlp = (MsgInfoList *) realloc ( p->mlp,
-                    (p->msg_count += ALLOC_MSGS)*sizeof(MsgInfoList) );
+                                           p->msg_count * sizeof(MsgInfoList) );
         if ( p->mlp == NULL)  {
             p->msg_count = 0;
             pop_log ( p, POP_PRIORITY, HERE,
@@ -750,7 +782,8 @@ CopyOneBull ( POP *p, long bnum, char *name )
     p->drop_size += strlen(buffer);
     mp->lines++;
 
-    sprintf ( buffer, "X-UIDL: %ld.%03d\n", timestamp, sequence );
+    sprintf ( mp->uidl_str, "%ld.%03d\n", timestamp, sequence++ );
+    sprintf ( buffer, "X-UIDL: %s", mp->uidl_str );
     if ( fputs ( buffer, p->drop ) == EOF ) {
         pop_log ( p, POP_PRIORITY, HERE, ERRMESG, 
                   name, p->temp_drop, 
@@ -760,16 +793,14 @@ CopyOneBull ( POP *p, long bnum, char *name )
 
     mp->length   += strlen ( buffer );
     p->drop_size += mp->length;
-    mp->lines++;
+    mp->lines    ++;
 
     DEBUG_LOG5 ( p, "Bull msg %lu (as %d [vis: %d]) being added to "
                     "list, offset %lu: %s",
                  bnum, mp->number, mp->visible_num, mp->offset, name );
-    sprintf ( buffer, "%ld.%03d\n", timestamp, sequence++ );
+    
 
-    mp->uidl_str = (char *) strdup ( buffer );
-
-    first_line = 0;
+    first_line = FALSE;
 
     while ( fgets ( buffer, MAXMSGLINELEN, bull ) != NULL ) {
         nchar = strlen(buffer);
@@ -786,7 +817,7 @@ CopyOneBull ( POP *p, long bnum, char *name )
                 continue;
 
             } else if ( *buffer == '\n' ) {
-                in_header = 0;
+                in_header = FALSE;
                 mp->body_lines = 1; /* Count new line as first line of body */
             }
         }
@@ -795,9 +826,9 @@ CopyOneBull ( POP *p, long bnum, char *name )
             msg_ends_in_nl = ( *buffer == '\n' );
         }
 
-        mp->length += nchar + 1; /* +1 is for CRLF */
+        mp->length   += nchar + 1; /* +1 is for CRLF */
         p->drop_size += nchar + 1;
-        mp->lines++;
+        mp->lines    ++;
 
         if ( fputs ( buffer, p->drop ) == EOF ) {
             pop_log ( p, POP_PRIORITY, HERE, ERRMESG, 
