@@ -5,13 +5,22 @@
 
 
 /*
- * Copyright (c) 2000 Qualcomm Incorporated.  All rights reserved.
+ * Copyright (c) 2005 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
  *
  * Revisions:
  *
+ *    4/11/05  [Randall Gellens]
+ *           - Open trace file as user, not root (fix security hole
+ *             reported by Jens Steube)
+ *           - Set umask to make sure we don't create group- or world-
+ *             writable files (e.g., trace or db)
+ *
+ *    7/29/03  [Randall Gellens]
+ *           - Removed prototype for srandom(); use stdlib.h instead.
+ *
  *   01/02/00  [Randall Gellens]
  *           - Added checks for dbm errors.
  *
@@ -126,8 +135,6 @@
 #  include <stdlib.h>
 #  define srandom srand
 #  define  random  rand
-#else
-   extern void srandom();
 #endif /* HAVE_SRANDOM */
 
 #include <flock.h>
@@ -154,6 +161,14 @@ extern int decode64();
 char *strerror();
 #endif /* HAVE_STRERROR */
 
+#ifdef HAVE_SYS_UNISTD_H
+#  include <sys/unistd.h>
+#endif /* HAVE_SYS_UNISTD_H */
+
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif /* include <unistd.h> */
+
 #include "popper.h"
 #include "logit.h"
 
@@ -211,7 +226,7 @@ static struct swit {
     { "dump"    , 0, DUMPSW  },
 #endif /* _DEBUG */
 
-    { NULL        },
+    { NULL      , 0, BAD_SW  },
 };
 
 
@@ -277,6 +292,7 @@ static void         adios   ( WHENCE, const char *fmt, ... );
 static void         helpful ( void );
 static int          check_db_err ( void *db, const char *op, BOOL bExp );
 static const char  *printable ( const char *p, int len );
+static void         open_trace ( char *fname );
 
 
 static void
@@ -452,6 +468,29 @@ printable ( const char *p, int len )
     return printable_buffer;
 }
 
+void
+open_trace ( char *tname )
+{
+    UID_T uid_save = -1;
+    UID_T myuid    = -1;
+
+
+    uid_save = geteuid();
+    myuid = getuid();
+    if ( seteuid ( myuid ) != 0 )
+        adios ( HERE, "internal error @ %i", __LINE__ );
+    
+    trace_file = fopen ( tname, "a+" );
+    if ( trace_file == NULL )
+        adios ( HERE, "Unable to open trace file \"%s\": %s (%d)\n",
+                tname, STRERROR(errno), errno );
+    BLATHER1 ( "Trace and Debug destination is file \"%s\"",
+               tname );
+            
+    if ( seteuid ( uid_save ) != 0 )
+        adios ( HERE, "internal error @ %i", __LINE__ );
+}
+
 
 #ifndef HAVE_STRDUP
 #include <stddef.h>
@@ -515,14 +554,6 @@ char *getpass();
 #  include <stdio.h>
 #  include <termios.h>
 
-#  ifdef HAVE_SYS_UNISTD_H
-#    include <sys/unistd.h>
-#  endif /* HAVE_SYS_UNISTD_H */
-
-#  ifdef HAVE_UNISTD_H
-#    include <unistd.h>
-#  endif /* include <unistd.h> */
-
 #  ifndef TCSASOFT
 #    define TCSASOFT 0
 #  endif
@@ -669,6 +700,7 @@ char   *argv[];
 
     memset  ( &pop_pw, 0, sizeof(pop_pw) );
     memset  ( &my_pw,  0, sizeof(my_pw)  );
+    umask   ( 0077 ); /* make sure we don't create group- or world-writable files */
     srandom ( (unsigned int) time ( (TIME_T *) 0) );   /* seed random with the
                                                           current time */
 
@@ -748,13 +780,7 @@ char   *argv[];
                 helpful();
             case TRACESW:
                 debug++;
-                trace_file = fopen ( argv[1], "a+" );
-                if ( trace_file == NULL )
-                    adios ( HERE, 
-                            "Unable to open trace file \"%s\": %s (%d)\n",
-                            argv[1], STRERROR(errno), errno );
-                BLATHER1 ( "Trace and Debug destination is file \"%s\"",
-                           argv[1] );
+                open_trace ( argv[1] );
                 argc--;
                 argv++;
                 break;
